@@ -15,7 +15,8 @@ func GetLans() ([]*net.IPNet, error) {
 	if err != nil {
 		return nil, err
 	}
-	addrs := []net.Addr{}
+	var addrs []net.Addr
+
 	for _, currentIf := range ifs {
 		if currentIf.Flags&net.FlagUp != net.FlagUp {
 			continue
